"""
Materialized Feature Store package.

Copyright (c) 2025 Faustino Lopez Ramos.
For licensing information, see the LICENSE file in the project root
"""
from core.feature_store.materialized.store import MaterializedFeatureStore

__all__ = ["MaterializedFeatureStore"]
