"""
Hybrid Feature Store - Combines materialized and virtualized paradigms.

Copyright (c) 2025 Faustino Lopez Ramos.
For licensing information, see the LICENSE file in the project root
"""

from core.feature_store.hybrid.store import HybridFeatureStore

__all__ = ["HybridFeatureStore"]
